# -*- coding: utf-8 -*-
import datetime
import os
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User

from django.conf import settings
from django.db.models import Q

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.i18n.models import TranslationModel, I18NModel
from jmb.core.db.fields import HTMLFieldTinyMCE
from jmb.core.middleware.thread_local import get_current_user, get_request





##################################
# CATEGORIA NEWS                 #
##################################
class CategoryManager(models.Manager):
    def get_query_set(self):
        qs = super(CategoryManager, self).get_query_set()
        return qs.filter(status=1)

class AllCategoryManager(models.Manager):
    def get_query_set(self):
        return super(CategoryManager, self).get_query_set()

class Category(DateModel, UserModel, StatusModel, OrderedModel):

    name = models.CharField(
        _("Name"),
        db_index=True,
        max_length=200,
    )
    slug = models.SlugField(
        _("Slug"),
        db_index=True,
        max_length=200,
    )

    objects = CategoryManager()
    all_objects = AllCategoryManager()

    def __unicode__(self):
        return u"%s" % self.name


    class Meta:
        verbose_name = _("Category")
        verbose_name_plural = _("News Categories")

    def get_absolute_url(self):
        return reverse('news_category_detail', args=[self.slug])





class CategoryTranslation(TranslationModel):
    parent = models.ForeignKey(
        Category, db_index=True,
        related_name='translations',
        verbose_name=_('parent'),
    )
    name = models.CharField(
        max_length=200,
        verbose_name=_('Name'),
    )



    class Meta:
        verbose_name = _('Category Translation')
        verbose_name_plural = _('Category Translations')
        unique_together = (('parent', 'language',),)

    def __unicode__(self):
        return u'%s' % self.name[:15]

###################################
## NEWS                           #
###################################
def get_news_upload_to(instance, filename):
    return "%s/%s/" % (
        settings.NEWS_PATH, instance.event_date
    )


class NewsManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        qs = super(NewsManager, self).get_query_set()
        qs = qs.filter(status=1)
        if user.is_superuser:
            return qs
        if not user.is_authenticated():
            return qs.filter(users=None)
        else:
            q1 = Q(users = None)
            q2 = Q(users = user)
            return qs.filter(q1 | q2)


class AllNewsManager(models.Manager):
    def get_query_set(self):
        return super(NewsManager, self).get_query_set()


class News(DateModel, UserModel, StatusModel, OrderedModel, I18NModel, SEOModel):
    categories = models.ManyToManyField(
        Category,
        null=True,
        blank=True,
        related_name="news",
        verbose_name=_('Categories')
    )
    users = models.ManyToManyField(
        User,
        blank=True,
        null=True,
        related_name='news',
        verbose_name=_('Visible only to'),
        help_text=_('News visible to these users, if empty is visible to all users.'),
    )
    slug = models.SlugField(
            verbose_name=_('Slug'),
            unique=True,
    )
    event_date = models.DateField(
        default= datetime.date.today,
        verbose_name=_('News date'),
        help_text=_('Date which news refers to.'),
    )
    pub_date_begin = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date begin'),
        help_text=_('When news publication date begins.'),
    )
    pub_date_end = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date end'),
        help_text=_('When news publication date ends.'),
    )
    title = models.CharField(
        max_length=255,
        verbose_name=_('Default title'),
    )
    small_image = models.ImageField(
        upload_to='thumbnail',
        blank=True, null=True,
        verbose_name=_("Small Image")
    )
    big_image = models.ImageField(
        upload_to='bigimage',
        blank=True, null=True,
        verbose_name=_("Big Image")
    )
    short_description = models.CharField(
        max_length=100,
        verbose_name=_('Short Description'),
        blank=True, null=True
    )
    full_description = HTMLFieldTinyMCE(
        blank=True, null=True,
        verbose_name=_("Full description"),
        help_text=_("Full description"),
    )
    note = models.TextField(
        blank=True,
        null=True,
        verbose_name=_('Note'),
    )
    file = models.FileField(
        upload_to=get_news_upload_to,
        verbose_name=_("File"),
        blank=True,
        null=True,
    )
    link = models.URLField(
        blank=True, null=True,
        verify_exists=False,
        help_text=_("If not blank, override the default's news url")
    )

    objects = NewsManager()
    all_objects = AllNewsManager()

    class Meta(object):
        verbose_name = _('News')
        verbose_name_plural = _('News')

    def __unicode__(self):
        return self.title[:15]

    def get_absolute_url(self):
        url = reverse('news_news_detail', args=[self.slug])
        return url

    def get_users(self):
        users = self.users.all().order_by('username')
        if users:
            return u', '.join([user.username for user in users])
        else:
            return _(u'All')
    get_users.short_description = _('show to')

    def get_categories(self):
        categories = self.categories.all().order_by('name')
        if categories:
            return u', '.join([category.name for category in categories])
        else:
            return _(u'All')
    get_categories.short_description = _('Categories')


    def get_short_description(self):
        return self.short_description or "%s ....." % self.text[:100]

    def get_filename_uno(self):
        return os.path.basename(self.file.name)




class NewsTranslation(TranslationModel):
    parent = models.ForeignKey(
        News,
        db_index=True,
        related_name='translations',
        verbose_name=_('parent'),
    )
    title = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('Default title'),
    )
    slug = models.SlugField(
        verbose_name=_('Slug'),
        unique=True,
    )
    short_description = models.CharField(
        max_length=100,
        verbose_name=_('Short Description'),
        blank=True, null=True
    )
    full_description = HTMLFieldTinyMCE(
        blank=True,
        null=True,
        verbose_name=_('Full Description'),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_('Note'),
    )

    class Meta:
        verbose_name = _('News Translation')
        verbose_name_plural = _('News Translations')
        unique_together = (('parent', 'language',),)

    def __unicode__(self):
        return u'%s (%s)' % (self.news.title, self.language)



#import cms stuffs
if settings.CMS_ENABLED:
    import jmb.news.cms.plugins
    import jmb.news.cms.app
